var _this = this;
import { createElement } from './create-element';
import { defaultTimeoutSeconds } from './option';
var loadingPromiseMaps = [];
var tryClearTimeout = function (timeoutId) {
    if (timeoutId) {
        clearTimeout(timeoutId);
    }
};
var findLoadingPromise = function (id, url) {
    var map = loadingPromiseMaps.find(function (x) { return (id && x[0] === id) || x[0] === url; });
    var _a = map !== null && map !== void 0 ? map : [], promise = _a[1];
    return promise;
};
var removeFromLoadingPromiseMap = function (id, url) {
    var idx = loadingPromiseMaps.findIndex(function (x) { return (id && x[0] === id) || x[0] === url; });
    if (idx >= 0) {
        loadingPromiseMaps.splice(idx, 1);
    }
};
var loadResource = function (_a) {
    var _b, _c;
    var url = _a.url, id = _a.id, timeoutSeconds = _a.timeoutSeconds, appendTo = _a.appendTo, hostElementTagName = _a.hostElementTagName, beforeElementCreateCallback = _a.beforeElementCreateCallback;
    var _document = appendTo ? appendTo.ownerDocument : document;
    var _window = (_c = (_b = _document.defaultView) !== null && _b !== void 0 ? _b : _document.parentWindow) !== null && _c !== void 0 ? _c : window;
    var isDifferentCtx = _document !== document;
    var promise = findLoadingPromise(isDifferentCtx ? 'load-resource-to-another-window-context-' + _window.name + '-' + id : id, url);
    if (promise)
        return promise;
    var loadingEl = _document.querySelector("".concat(hostElementTagName, "#").concat(id));
    loadingEl = !loadingEl
        ? hostElementTagName === 'script' && _document.querySelector("script[src=\"".concat(url, "\"]"))
        : loadingEl;
    loadingEl = !loadingEl
        ? hostElementTagName === 'link' && _document.querySelector("link[href=\"".concat(url, "\"]"))
        : loadingEl;
    if (loadingEl)
        return Promise.resolve(loadingEl);
    promise = new Promise(function (resolve, reject) {
        createElement(hostElementTagName, function (el) {
            el.id = id !== null && id !== void 0 ? id : '';
            appendTo !== null && appendTo !== void 0 ? appendTo : (appendTo = _document.head);
            timeoutSeconds = timeoutSeconds <= 0 ? defaultTimeoutSeconds : timeoutSeconds;
            if (el instanceof HTMLLinkElement) {
                el.href = url;
                el.rel = 'stylesheet';
                el.type = 'text/css';
            }
            else if (el instanceof HTMLScriptElement) {
                el.src = url;
                el.type = 'text/javascript';
                el.defer = true;
            }
            var timeoutId;
            el.onload = function () {
                tryClearTimeout(timeoutId);
                removeFromLoadingPromiseMap(id, url);
                resolve(el);
            };
            el.onerror = function (ev) {
                tryClearTimeout(timeoutId);
                el.remove();
                removeFromLoadingPromiseMap(id, url);
                reject(ev);
            };
            beforeElementCreateCallback === null || beforeElementCreateCallback === void 0 ? void 0 : beforeElementCreateCallback.call(_this, el);
            appendTo.append(el);
            timeoutId = setTimeout(function () {
                tryClearTimeout(timeoutId);
                el.remove();
                removeFromLoadingPromiseMap(id, url);
                reject("timeout after ".concat(timeoutSeconds, " seconds"));
            }, timeoutSeconds * 1000);
        });
    });
    loadingPromiseMaps.push([id ? id : url, promise]);
    return promise;
};
export { loadResource };
